@@ -1,6 +1,6 @@
 /*
  * Deskflow -- mouse and keyboard sharing utility
- * SPDX-FileCopyrightText: (C) 2025 Deskflow Developers
+ * SPDX-FileCopyrightText: (C) 2025 - 2026 Deskflow Developers
  * SPDX-FileCopyrightText: (C) 2012 Symless Ltd.
  * SPDX-FileCopyrightText: (C) 2002 Chris Schoeneman
  * SPDX-License-Identifier: GPL-2.0-only WITH LicenseRef-OpenSSL-Exception
@@ -11,13 +11,13 @@
 #include "arch/Arch.h"
 #include "base/IEventQueue.h"
 #include "base/Log.h"
-#include "base/Path.h"
 #include "common/ExitCodes.h"
+#include "common/PlatformInfo.h"
+#include "common/Settings.h"
 #include "deskflow/App.h"
-#include "deskflow/ArgParser.h"
+#include "deskflow/ProtocolTypes.h"
 #include "deskflow/Screen.h"
 #include "deskflow/ScreenException.h"
-#include "deskflow/ServerArgs.h"
 #include "net/SocketException.h"
 #include "net/SocketMultiplexer.h"
 #include "net/TCPSocketFactory.h"
@@ -30,10 +30,6 @@
 // must be before screen header includes
 #include <QFileInfo>
 
-#if SYSAPI_WIN32
-#include "arch/win32/ArchMiscWindows.h"
-#endif
-
 #if WINAPI_MSWINDOWS
 #include "platform/MSWindowsScreen.h"
 #endif
@@ -53,14 +49,7 @@
 #include "platform/OSXScreen.h"
 #endif
 
-#if defined(WINAPI_XWINDOWS) or defined(WINAPI_LIBEI)
-#include "platform/Wayland.h"
-#endif
-
 #include <fstream>
-#include <iostream>
-#include <sstream>
-#include <stdio.h>
 
 using namespace deskflow::server;
 
@@ -68,74 +57,26 @@ using namespace deskflow::server;
 // ServerApp
 //
 
-ServerApp::ServerApp(IEventQueue *events) : App(events, new deskflow::ServerArgs())
+ServerApp::ServerApp(IEventQueue *events, const QString &processName) : App(events, processName)
 {
+  m_name = Settings::value(Settings::Core::ComputerName).toString().toStdString();
   // do nothing
 }
 
-void ServerApp::parseArgs(int argc, const char *const *argv)
+void ServerApp::parseArgs()
 {
-
-  ArgParser argParser(this);
-  bool result = argParser.parseServerArgs(args(), argc, argv);
-
-  if (!result || args().m_shouldExitOk || args().m_shouldExitFail) {
-    if (args().m_shouldExitOk) {
-      bye(s_exitSuccess);
-    } else {
-      bye(s_exitArgs);
-    }
+  if (const auto address = Settings::value(Settings::Core::Interface).toString(); !address.isEmpty()) {
+    *m_deskflowAddress = NetworkAddress(address.toStdString(), Settings::value(Settings::Core::Port).toInt());
   } else {
-    if (!args().m_deskflowAddress.empty()) {
-      try {
-        *m_deskflowAddress = NetworkAddress(args().m_deskflowAddress, kDefaultPort);
-        m_deskflowAddress->resolve();
-      } catch (SocketAddressException &e) {
-        LOG_CRIT("%s: %s" BYE, args().m_pname, e.what(), args().m_pname);
-        bye(s_exitArgs);
-      }
-    }
+    *m_deskflowAddress = NetworkAddress(Settings::value(Settings::Core::Port).toInt());
   }
-}
 
-void ServerApp::help()
-{
-  std::stringstream help;
-  help << "\n\nServer Mode:\n\n"
-       << "Usage: " << kAppId << "-core server"
-       << " --config <pathname>"
-       << " [--address <address>]"
-
-#if WINAPI_XWINDOWS
-       << " [--display <display>]"
-#endif
-
-       << s_helpSysArgs << s_helpCommonArgs << "\n"
-       << "  -a, --address <address>  listen for clients on the given address.\n"
-       << "  -c, --config <pathname>  path of the configuration file\n"
-       << s_helpGeneralArgs
-       << "      --disable-client-cert-check disable client SSL certificate \n"
-          "                                     checking (deprecated)\n"
-       << s_helpSysInfo << s_helpVersionArgs << "\n"
-
-#if WINAPI_XWINDOWS
-       << "      --display <display>  when in X mode, connect to the X server\n"
-       << "                             at <display>.\n"
-#endif
-
-       << "* marks defaults.\n"
-
-       << s_helpNoWayland
-
-       << "\n"
-       << "The argument for --address is of the form: [<hostname>][:<port>].  "
-          "The\n"
-       << "hostname must be the address or hostname of an interface on the "
-       << "system.\n"
-       << "The default is to listen on all interfaces.  The port overrides the\n"
-       << "default port, " << kDefaultPort << ".\n";
-
-  LOG_PRINT("%s", help.str().c_str());
+  try {
+    m_deskflowAddress->resolve();
+  } catch (SocketAddressException &e) {
+    LOG_CRIT("%s: %s" BYE, qPrintable(processName()), e.what(), qPrintable(processName()));
+    bye(s_exitArgs);
+  }
 }
 
 void ServerApp::reloadSignalHandler(Arch::ThreadSignal, void *)
@@ -144,12 +85,19 @@ void ServerApp::reloadSignalHandler(Arch::ThreadSignal, void *)
   events->addEvent(Event(EventTypes::ServerAppReloadConfig, events->getSystemTarget()));
 }
 
+QString ServerApp::currentConfig() const
+{
+  bool useExt = Settings::value(Settings::Server::ExternalConfig).toBool();
+  return useExt ? Settings::value(Settings::Server::ExternalConfigFile).toString()
+                : Settings::defaultValue(Settings::Server::ExternalConfigFile).toString();
+}
+
 void ServerApp::reloadConfig()
 {
   LOG_DEBUG("reload configuration");
-  if (loadConfig(args().m_configFile)) {
+  if (loadConfig(currentConfig())) {
     if (m_server != nullptr) {
-      m_server->setConfig(*args().m_config);
+      m_server->setConfig(*m_config);
     }
     LOG_NOTE("reloaded configuration");
   }
@@ -157,34 +105,39 @@ void ServerApp::reloadConfig()
 
 void ServerApp::loadConfig()
 {
-  const auto path = args().m_configFile;
-  if (path.empty()) {
+  const auto path = currentConfig();
+  if (path.isEmpty()) {
     LOG_CRIT("no configuration path provided");
     bye(s_exitConfig);
   }
 
   if (!loadConfig(path)) {
-    LOG_CRIT("%s: failed to load config: %s", args().m_pname, path.c_str());
+    LOG_CRIT("%s: failed to load config: %s", qPrintable(processName()), path.toStdString().c_str());
     bye(s_exitConfig);
   }
 }
 
-bool ServerApp::loadConfig(const std::string &pathname)
+bool ServerApp::loadConfig(const QString &filename)
 {
+  const auto path = filename.toStdString();
   try {
     // load configuration
-    LOG_DEBUG("opening configuration \"%s\"", pathname.c_str());
-    std::ifstream configStream(deskflow::filesystem::path(pathname));
+    LOG_DEBUG("opening configuration \"%s\"", path.c_str());
+#ifdef SYSAPI_WIN32
+    std::ifstream configStream(filename.toStdWString());
+#else
+    std::ifstream configStream(path);
+#endif
     if (!configStream.is_open()) {
-      LOG_ERR("cannot open configuration \"%s\"", pathname.c_str());
+      LOG_ERR("cannot open configuration \"%s\"", path.c_str());
       return false;
     }
-    configStream >> *args().m_config;
+    configStream >> *m_config;
     LOG_DEBUG("configuration read successfully");
     return true;
   } catch (ServerConfigReadException &e) {
     // report error in configuration file
-    LOG_ERR("cannot read configuration \"%s\": %s", pathname.c_str(), e.what());
+    LOG_ERR("cannot read configuration \"%s\": %s", path.c_str(), e.what());
   }
   return false;
 }
@@ -360,7 +313,7 @@ bool ServerApp::initServer()
   deskflow::Screen *serverScreen = nullptr;
   PrimaryClient *primaryClient = nullptr;
   try {
-    std::string name = args().m_config->getCanonicalName(args().m_name);
+    std::string name = m_config->getCanonicalName(m_name);
     serverScreen = openServerScreen();
     primaryClient = openPrimaryClient(name, serverScreen);
     m_serverScreen = serverScreen;
@@ -384,18 +337,7 @@ bool ServerApp::initServer()
     return false;
   }
 
-  if (args().m_restartable) {
-    // install a timer and handler to retry later
-    assert(m_timer == nullptr);
-    LOG_DEBUG("retry in %.0f seconds", retryTime);
-    m_timer = getEvents()->newOneShotTimer(retryTime, nullptr);
-    getEvents()->addHandler(EventTypes::Timer, m_timer, [this](const auto &) { retryHandler(); });
-    m_serverState = Initializing;
-    return true;
-  } else {
-    // don't try again
-    return false;
-  }
+  return false;
 }
 
 deskflow::Screen *ServerApp::openServerScreen()
@@ -435,8 +377,8 @@ bool ServerApp::startServer()
 
   ClientListener *listener = nullptr;
   try {
-    listener = openClientListener(args().m_config->getDeskflowAddress());
-    m_server = openServer(*args().m_config, m_primaryClient);
+    listener = openClientListener(m_config->getDeskflowAddress());
+    m_server = openServer(*m_config, m_primaryClient);
     listener->setServer(m_server);
     m_server->setListener(listener);
     m_listener = listener;
@@ -444,11 +386,7 @@ bool ServerApp::startServer()
     m_serverState = Started;
     return true;
   } catch (SocketAddressInUseException &e) {
-    if (args().m_restartable) {
-      LOG_ERR("cannot listen for clients: %s", e.what());
-    } else {
-      LOG_CRIT("cannot listen for clients: %s", e.what());
-    }
+    LOG_CRIT("cannot listen for clients: %s", e.what());
     closeClientListener(listener);
   } catch (BaseException &e) {
     LOG_CRIT("failed to start server: %s", e.what());
@@ -456,25 +394,15 @@ bool ServerApp::startServer()
     return false;
   }
 
-  if (args().m_restartable) {
-    // install a timer and handler to retry later
-    assert(m_timer == nullptr);
-    const auto retryTime = 10.0;
-    LOG_DEBUG("retry in %.0f seconds", retryTime);
-    m_timer = getEvents()->newOneShotTimer(retryTime, nullptr);
-    getEvents()->addHandler(EventTypes::Timer, m_timer, [this](const auto &) { retryHandler(); });
-    m_serverState = Starting;
-    return true;
-  } else {
-    // don't try again
-    return false;
-  }
+  return false;
 }
 
 deskflow::Screen *ServerApp::createScreen()
 {
 #if WINAPI_MSWINDOWS
-  return new deskflow::Screen(new MSWindowsScreen(true, args().m_noHooks, getEvents()), getEvents());
+  return new deskflow::Screen(
+      new MSWindowsScreen(true, Settings::value(Settings::Core::UseHooks).toBool(), getEvents()), getEvents()
+  );
 #endif
 
 #if defined(WINAPI_XWINDOWS) or defined(WINAPI_LIBEI)
@@ -490,7 +418,10 @@ deskflow::Screen *ServerApp::createScreen()
 
 #if WINAPI_XWINDOWS
   LOG_INFO("using legacy x windows screen");
-  return new deskflow::Screen(new XWindowsScreen(args().m_display, true, 0, getEvents()), getEvents());
+  return new deskflow::Screen(
+      new XWindowsScreen(qPrintable(Settings::value(Settings::Core::Display).toString()), true, getEvents()),
+      getEvents()
+  );
 #elif WINAPI_CARBON
   return new deskflow::Screen(new OSXScreen(getEvents(), true), getEvents());
 #endif
@@ -524,8 +455,8 @@ ClientListener *ServerApp::openClientListener(const NetworkAddress &address)
 {
   using enum SecurityLevel;
   auto securityLevel = PlainText;
-  if (args().m_enableCrypto) {
-    if (args().m_chkPeerCert) {
+  if (Settings::value(Settings::Security::TlsEnabled).toBool()) {
+    if (Settings::value(Settings::Security::CheckPeers).toBool()) {
       securityLevel = PeerAuth;
     } else {
       securityLevel = Encrypted;
@@ -543,7 +474,7 @@ ClientListener *ServerApp::openClientListener(const NetworkAddress &address)
 
 Server *ServerApp::openServer(ServerConfig &config, PrimaryClient *primaryClient)
 {
-  auto *server = new Server(config, primaryClient, m_serverScreen, getEvents(), args());
+  auto *server = new Server(config, primaryClient, m_serverScreen, getEvents());
   try {
     getEvents()->addHandler(EventTypes::ServerScreenSwitched, server, [this](const auto &) { handleScreenSwitched(); });
 
@@ -578,22 +509,22 @@ int ServerApp::mainLoop()
 
   // if configuration has no screens then add this system
   // as the default
-  if (args().m_config->begin() == args().m_config->end()) {
-    args().m_config->addScreen(args().m_name);
+  if (m_config->begin() == m_config->end()) {
+    m_config->addScreen(m_name);
   }
 
   // set the contact address, if provided, in the config.
   // otherwise, if the config doesn't have an address, use
   // the default.
   if (m_deskflowAddress->isValid()) {
-    args().m_config->setDeskflowAddress(*m_deskflowAddress);
-  } else if (!args().m_config->getDeskflowAddress().isValid()) {
-    args().m_config->setDeskflowAddress(NetworkAddress(kDefaultPort));
+    m_config->setDeskflowAddress(*m_deskflowAddress);
+  } else if (!m_config->getDeskflowAddress().isValid()) {
+    m_config->setDeskflowAddress(NetworkAddress(kDefaultPort));
   }
 
   // canonicalize the primary screen name
-  if (std::string primaryName = args().m_config->getCanonicalName(args().m_name); primaryName.empty()) {
-    LOG_CRIT("unknown screen name `%s'", args().m_name.c_str());
+  if (std::string primaryName = m_config->getCanonicalName(m_name); primaryName.empty()) {
+    LOG_CRIT("unknown screen name `%s'", m_name.c_str());
     return s_exitFailed;
   }
 
@@ -621,7 +552,6 @@ int ServerApp::mainLoop()
   // run event loop.  if startServer() failed we're supposed to retry
   // later.  the timer installed by startServer() will take care of
   // that.
-  DAEMON_RUNNING(true);
 
 #if WINAPI_CARBON
 
@@ -636,10 +566,8 @@ int ServerApp::mainLoop()
   getEvents()->loop();
 #endif
 
-  DAEMON_RUNNING(false);
-
   // close down
-  LOG_DEBUG1("stopping server");
+  LOG_DEBUG("stopping server");
   getEvents()->removeHandler(EventTypes::ServerAppForceReconnect, getEvents()->getSystemTarget());
   getEvents()->removeHandler(EventTypes::ServerAppReloadConfig, getEvents()->getSystemTarget());
   cleanupServer();
@@ -656,61 +584,30 @@ void ServerApp::resetServer()
   startServer();
 }
 
-int ServerApp::runInner(int argc, char **argv, StartupFunc startup)
+int ServerApp::runInner(StartupFunc startup)
 {
   // general initialization
   m_deskflowAddress = new NetworkAddress;
-  args().m_config = std::make_shared<Config>(getEvents());
-  args().m_pname = QFileInfo(argv[0]).fileName().toLocal8Bit().constData();
+  m_config = std::make_shared<Config>(getEvents());
 
   // run
-  int result = startup(argc, argv);
+  int result = startup();
 
   delete m_deskflowAddress;
   return result;
 }
 
-int daemonMainLoopStatic(int argc, const char **argv)
+int ServerApp::start()
 {
-  return ServerApp::instance().daemonMainLoop(argc, argv);
-}
-
-int ServerApp::standardStartup(int argc, char **argv)
-{
-  initApp(argc, argv);
-
-  // daemonize if requested
-  if (args().m_daemon) {
-    return ARCH->daemonize(daemonName(), daemonMainLoopStatic);
-  } else {
-    return mainLoop();
-  }
-}
-
-int ServerApp::foregroundStartup(int argc, char **argv)
-{
-  initApp(argc, argv);
-
-  // never daemonize
+  initApp();
   return mainLoop();
 }
 
 const char *ServerApp::daemonName() const
 {
-#if SYSAPI_WIN32
-  return "Deskflow Server";
-#elif SYSAPI_UNIX
+  if (deskflow::platform::isWindows())
+    return "Deskflow Server";
   return "deskflow-server";
-#endif
-}
-
-const char *ServerApp::daemonInfo() const
-{
-#if SYSAPI_WIN32
-  return "Shares this computers mouse and keyboard with other computers.";
-#elif SYSAPI_UNIX
-  return "";
-#endif
 }
 
 void ServerApp::startNode()
