@@ -1,5 +1,6 @@
 /*
  * Deskflow -- mouse and keyboard sharing utility
+ * SPDX-FileCopyrightText: (C) 2026 Deskflow Developers
  * SPDX-FileCopyrightText: (C) 2025 Symless Ltd.
  * SPDX-License-Identifier: GPL-2.0-only WITH LicenseRef-OpenSSL-Exception
  */
@@ -18,12 +19,13 @@ class FileTail : public QObject
   Q_OBJECT
 
 public:
-  FileTail(const QString &filePath, QObject *parent = nullptr);
+  explicit FileTail(const QString &filePath, QObject *parent = nullptr);
+  void setWatchedFile(const QString &filePath);
 
-signals:
+Q_SIGNALS:
   void newLine(const QString &line);
 
-private slots:
+private Q_SLOTS:
   void handleFileChanged(const QString &);
 
 private:
