@@ -6,9 +6,13 @@
  */
 
 #include "platform/MSWindowsHook.h"
+#include "base/DirectionTypes.h"
 #include "base/Log.h"
-#include "deskflow/ProtocolTypes.h"
-#include "deskflow/XScreen.h"
+#include "deskflow/ScreenException.h"
+
+#ifndef WM_MOUSEHWHEEL
+#define WM_MOUSEHWHEEL 0x020E
+#endif
 
 static const char *g_name = "dfwhook";
 
@@ -53,9 +57,9 @@ void MSWindowsHook::loadLibrary()
 
   // initialize library
   if (init(GetCurrentThreadId()) == 0) {
-    LOG((CLOG_ERR "failed to init %s.dll, another program may be using it", g_name));
-    LOG((CLOG_INFO "restarting your computer may solve this error"));
-    throw XScreenOpenFailure();
+    LOG_ERR("failed to init %s.dll, another program may be using it", g_name);
+    LOG_INFO("restarting your computer may solve this error");
+    throw ScreenOpenFailureException();
   }
 }
 
@@ -505,6 +509,13 @@ static bool mouseHookHandler(WPARAM wParam, int32_t x, int32_t y, int32_t data)
     }
     return (g_mode == kHOOK_RELAY_EVENTS);
 
+  case WM_MOUSEHWHEEL:
+    if (g_mode == kHOOK_RELAY_EVENTS) {
+      // relay event
+      PostThreadMessage(g_threadID, DESKFLOW_MSG_MOUSE_WHEEL, 0, data);
+    }
+    return (g_mode == kHOOK_RELAY_EVENTS);
+
   case WM_NCMOUSEMOVE:
   case WM_MOUSEMOVE:
     if (g_mode == kHOOK_RELAY_EVENTS) {
@@ -620,9 +631,10 @@ EHookResult MSWindowsHook::install()
     }
   }
 #endif
-
+  // clang-format off
   // check that we got all the hooks we wanted
-  if ((g_mouseLL == nullptr) ||
+  if (
+      (g_mouseLL == nullptr) ||
 #if !NO_GRAB_KEYBOARD
       (g_keyboardLL == nullptr)
 #endif
@@ -630,7 +642,7 @@ EHookResult MSWindowsHook::install()
     uninstall();
     return kHOOK_FAILED;
   }
-
+  // clang-format on
   if (g_keyboardLL != nullptr || g_mouseLL != nullptr) {
     g_hookThread = GetCurrentThreadId();
     return kHOOK_OKAY_LL;
